import datetime
from datetime import timezone
import math

import iteration_utilities
import pandas
from typing_extensions import Self, List, Dict, Any, TYPE_CHECKING
from inspect import isclass
from furthrmind.collection.baseclass import BaseClass
from furthrmind.collection.fielddata import FieldData
if TYPE_CHECKING:
    from furthrmind.collection import Unit


class Column(BaseClass):
    """
    Attributes
    ----------
    id : str
        id of the column
    name : str
        name of the column
    type : str
        Type of the column. Either "Text" or "Numeric"
    values : List[Any]
        "These represent the values held within the column. If the column type is 'Text',
        the values are characterized as strings. Conversely, for 'Numeric' type columns,
        the values are expressed as floating-point numbers."
    _fetched : bool
        This is a Boolean attribute indicating whether all attributes have been retrieved from the server or only
        the name and ID are present.
    """

    id: str = ""
    name: str = ""
    type: str = ""
    unit: "Unit" = None
    values: List[Any] = []

    _attr_definition = {"unit": {"class": "Unit"}}

    def __init__(self, id=None, data=None):
        super().__init__(id, data)

    def _update_attributes(self, data):
        super()._update_attributes(data)

        def convert_date(value):
            if not value:
                return value
            if isinstance(value, (int, float)):
                value = datetime.datetime.fromtimestamp(value)
            if isinstance(value, str):
                value = datetime.datetime.fromisoformat(value)
            return value

        if self.type == "Date":
            if self.values:
                self.values = list(map(convert_date, self.values))

    @classmethod
    def get(cls, id: str = "", project_id: str = "") -> Self:

        """
        Method to get one column by its id
        If called on an instance of the class, the id of the instance is used

        Parameters
        ----------
        id : str
            id of requested column
        project_id : str, optional
            Optionally to get a column from another project as the furthrmind sdk was initiated with, defaults to ""

        Returns
        -------
        Self
            Instance of column class

        Raises
        ------
        AssertionError
            If used as a class method and id is not specified.
        """

        if isclass(cls):
            assert id, "id must be specified"

        return cls._get(id, project_id=project_id)

    # noinspection PyMethodOverriding
    @classmethod
    def get_many(cls, ids: List[str] = (), project_id: str = "") -> List[Self]:
        """
        Method to get many columns belonging to one project

        Parameters
        ----------
        ids : List[str]
            List with ids.

        project_id : str
            Optionally, the id of the project from which to get the experiments. Defaults to an empty string.

        Returns
        -------
        List[Self]
            List with instances of the experiment class.

        Raises
        ------
        AssertionError
            If `ids` is not specified.
        """

        assert ids, "ids must be specified"
        return cls._get_many(ids, project_id=project_id)

    @classmethod
    def _get_all(cls, project_id=None) -> List[Self]:
        raise ValueError("Not implemented for columns")

    def _get_url_instance(self, project_id=None):
        project_url = Column.fm.get_project_url(project_id)
        url = f"{project_url}/column/{self.id}"
        return url

    @classmethod
    def _get_url_class(cls, id, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/column/{id}"
        return url

    @classmethod
    def _post_url(cls, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/column"
        return url

    @classmethod
    def _get_all_url(cls, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/column"
        return url

    @classmethod
    def _type_check(cls, column_type, data):
        column_type = column_type.capitalize()
        if not column_type in ["Text", "Numeric", "Date", "Bool"]:
            raise ValueError("Column type must be Text/Numeric/Date/Bool.")
        if isinstance(data, pandas.Series):
            data = data.tolist()

        data = list(map(Column._convert_nan, data))

        if column_type == "Text":
            if iteration_utilities.all_isinstance(data, (str, type(None))):
                return data
            return [str(d) for d in data]

        elif column_type == "Numeric":
            if iteration_utilities.all_isinstance(data, (int, float, type(None))):
                return data
            data = list(map(Column._convert_float, data))
            return data
        
        elif column_type == "Date":
            if iteration_utilities.all_isinstance(data, (int, float, type(None))):
                return data
            data = list(map(Column._convert_date, data))
            return data

    @staticmethod
    def _convert_nan(value):
        if value is None:
            return None
        try:
            if math.isnan(value):
                return None
            else:
                return value
        except:
            return value

    @staticmethod
    def _convert_float(value):
        if value is None:
            return value
        try:
            value = float(value)
            return value
        except:
            raise ValueError(
                "All column values must be a float, int or a string that can be converted to a float")

    @staticmethod
    def _convert_date(value):
        if value is None:
            return value
        if isinstance(value, datetime.datetime):
            if isinstance(value, pandas.Timestamp):
                value = value.to_pydatetime()
            if value.tzinfo is None:
                return value.isoformat()
            return int(value.timestamp())
        elif isinstance(value, datetime.date):
            value = datetime.datetime.combine(value, datetime.datetime.min.time())
            if value.tzinfo is None:
                return value.isoformat()
            return int(value.timestamp())
        elif isinstance(value, str):
            try:
                datetime.datetime.fromisoformat(value)
                return value
            except ValueError:
                raise TypeError("No iso time format")
        elif isinstance(value, (int, float)):
            return value
        raise ValueError("All column values must be a date, datetime, string, or int")

    @classmethod
    @BaseClass._create_instances_decorator(_fetched=False)
    def create(cls, name: str, type: str, data: list, unit=None, project_id=None) -> Self:
        """
        Method to create a new column

        Parameters
        ----------
        name : str
            Name of the column
        type : str
            The column type is categorized as either "Text" or "Numeric". For the "Text" type, all data
            will be transformed into strings. Conversely, for the "Numeric" type, data will be converted
            into floats, provided such a conversion is feasible. Please ensure that your data corresponds
            to the assigned column type.
        data : Union[list, pandas.Series]
            This should be either a list or a pandas series. Its values need to comply with the specified column type
            and will undergo conversion based on the rules described above.
        unit : Optional[Union[str, Dict]]
            Dict with id or name, or name as string, or id as string
        project_id : Optional[str]
            Optionally to create an item in another project as the furthrmind sdk was initiated with

        Returns
        -------
        Self
            Instance of column class

        """
        type = type.capitalize()
        data = cls._type_check(type, data)
        unit = FieldData._check_unit(unit)
        data_dict = {"name": name, "type": type, "values": data, "unit": unit}
        id = cls._post(data_dict, project_id)
        data_dict["id"] = id
        return data_dict

    @classmethod
    @BaseClass._create_instances_decorator(_fetched=False)
    def create_many(cls, data_list: List[Dict], project_id: str = "") -> List[Self]:
        """
        Method to create many new columns

        Parameters
        ----------
        data_list : List[Dict]
            A list of dictionaries containing information about the data columns to be created. Each dictionary should
             have the following keys:

                - name: Name of the column
                - type: Type of the column. Allowed values are "Text" or "Numeric".
                - data: List of column values. The values must match the column type. Can also be a pandas data series.
                - unit: Optional. Dictionary with id or name, or name as a string, or id as a string.

        project_id : str, optional
            Optionally to create columns in another project as the furthrmind sdk was initiated with

        Returns
        -------
        List[Self]
            A list of instances of the column class.

        """

        new_data_list = []
        for item in data_list:
            type:str = item.get("type", "")
            type = type.capitalize()
            unit = item.get("unit")
            name = item.get("name")
            data = item.get("value")
            if data is None:
                data = item.get("data")
            data = cls._type_check(type, data)
            unit = FieldData._check_unit(unit)
            data_dict = {"name": name, "type": type, "values": data, "unit": unit}
            new_data_list.append(data_dict)

        id_list = cls._post(new_data_list, project_id, force_list=True)
        for item, id in zip(new_data_list, id_list):
            item["id"] = id

        return new_data_list



from ..utils import furthr_wrap
from furthrmind.collection.baseclass import (BaseClassWithFieldData, BaseClassWithFiles,
                                             BaseClassWithGroup, BaseClass,
                                             BaseClassWithLinking, BaseClassWithNameUpdate, BaseClassWithProtected)
from typing_extensions import List, Dict, Self, TYPE_CHECKING
from inspect import isclass

if TYPE_CHECKING:
    from furthrmind.collection import File, FieldData, Sample, Group, Category, DataTable, Experiment

class ResearchItem(BaseClassWithFieldData, BaseClassWithFiles, BaseClassWithGroup, BaseClassWithLinking,
                   BaseClassWithNameUpdate, BaseClassWithProtected, BaseClass ):
    """
    Attributes
    ----------
    id : str
        id of the researchitem
    name : str
        name of the researchitem
    shortid : str
        shortid of the researchitem
    protected: bool
        Indicates, if the researchitem is protected in frontend. Does not protect the item for changes made by the api
    files : List[File]
        List of files belonging to this researchitem. See [File](file.md) for more information.
    fielddata : List[FieldData]
        List of field data belonging to this researchitem. See [FieldData](fielddata.md) for more information.
    linked_samples : List[Sample]
        This list contains 'sample' objects linked to the current researchitem. These objects are partially fetched,
        providing only the name and ID. To retrieve the entire object, invoke the 'get()' method on the 'sample'.
        Refer to nested objects in [Getting Started](index.md) for further details. For a comprehensive understanding of
        'sample' objects, see the provided [Sample](sample.md).
    linked_experiments : List[Experiment]
        This list contains 'experiment' objects linked to the current researchitem. These objects are partially fetched,
        providing only the name and ID. To retrieve the entire object, invoke the 'get()' method on the 'experiment'.
        Refer to nested objects in [Getting Started](index.md) for further details. For a comprehensive understanding of
        'experiment' objects, see the provided [Experiment](experiment.md).
    linked_researchitems : Dict[str, List[ResearchItem]]
        This is a dictionary with category name as keys and lists with the corresponding `researchitem` objects as values.
        The `researchitem` objects are partially fetched,
        providing only the name and ID. To retrieve the entire object, invoke the 'get()' method on the 'researchitem'.
        Refer to nested objects in [Getting Started](index.md) for further details. For a comprehensive understanding of
        'researchitem' objects, see the provided [ResearchItem](researchitem.md).
    groups : List[Group]
        This list contains 'group' objects the researchitem belongs to. These objects are partially fetched,
        providing only the name and ID. To retrieve the entire object, invoke the 'get()' method on the 'group'.
        Refer to nested objects in [Getting Started](index.md) for further details. For a comprehensive understanding of
        'group' objects, see the provided [Group](group.md).
    category : Category
        [Category](category.md) object of the researchitem.
    datatables : List[DataTable]
        This list contains 'datatable' objects that belong to this researchitem. These objects are partially fetched,
        providing only the name and ID. To retrieve the entire object, invoke the 'get()' method on the 'datatable'.
        Refer to nested objects in [Getting Started](index.md) for further details. For a comprehensive understanding of
        'datatable' objects, see the provided [DataTable](datatable.md).
    _fetched : bool
        This is a Boolean attribute indicating whether all attributes have been retrieved from the server or only
        the name and ID are present.
    """

    id = ""
    name = ""
    neglect = False
    neglect = False
    protected = False
    shortid = ""
    files: List["File"] = []
    fielddata: List["FieldData"] = []
    linked_experiments: List["Experiment"] = []
    linked_samples: List["Sample"] = []
    linked_researchitems: Dict[str, List["ResearchItem"]] = {}
    groups: List["Group"] = []
    category: "Category" = None
    datatables: List["DataTable"] = []

    _attr_definition = {
        "files": {"class": "File"},
        "fielddata": {"class": "FieldData"},
        "groups": {"class": "Group"},
        "linked_samples": {"class": "Sample"},
        "linked_experiments": {"class": "Experiment"},
        "linked_researchitems": {"class": "ResearchItem", "nested_dict": True},
        "datatables": {"class": "DataTable"},
        "category": {"class": "Category"}
    }

    def __init__(self, id=None, data=None):
        super().__init__(id, data)

    def _get_url_instance(self, project_id=None):
        project_url = ResearchItem.fm.get_project_url(project_id)
        url = f"{project_url}/researchitems/{self.id}"
        return url

    @classmethod
    def _get_url_class(cls, id, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/researchitems/{id}"
        return url

    @classmethod
    def _get_all_url(cls, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/researchitems"
        return url

    @classmethod
    def _post_url(cls, project_id=None):
        project_url = cls.fm.get_project_url(project_id)
        url = f"{project_url}/researchitems"
        return url
    
    @classmethod
    def get(cls, id: str = "", shortid: str = "", name: str = "", category_name: str = "", category_id: str = "",
            project_id: str = "") -> Self:
        """
        Method to get  one researchitem by its id, short_id or name. If requested by name, also category_name or
        category_id is required.
        If called on an instance of the class, the id of the class is used

        Parameters
        ----------
        id : str, optional
            The id of the requested research item.

        shortid : str, optional
            The short id of the requested research item.

        name : str, optional
            The name of the requested research item.

        category_name : str, optional
            The name of the category the research item belongs to.

        category_id : str, optional
            The id of the category the research item belongs to.

        project_id : str, optional
            Optionally to get a researchitem from another project as the furthrmind sdk was initiated with

        Returns
        -------
        Self
            Instance of the researchitem class.

        Raises
        ------
        AssertionError
            If used as a class method and neither id, shortid, nor name is provided.
            If used as a class method and name is provided but neither category_name nor category_id is provided.

        """

        if isclass(cls):
            assert id or shortid or name, AssertionError("Either id, shortid or name must be given")
            if not id and name:
                assert category_name or category_id, AssertionError("Either category name or id must be given")

        return cls._get(id=id, shortid=shortid, name=name,
                        category_name=category_name, category_id=category_id, project_id=project_id)


    @classmethod
    def get_many(cls, ids: List[str] = (), shortids: List[str] = (), names: List[str] = (),
                 category_name=None, category_id=None, project_id=None) -> List[Self]:
        """
        Method to get many researchitems by ids or shortids. If requested by names, also category_name or category_id
        is required. If requested by name items from one category can be requested at a time.

        Parameters
        ----------
        ids : List[str]
             List of experiment ids.
        shortids : List[str]
             List of experiment short ids.
        names : List[str]
             List of experiment names.
        category_name : str, optional
             Name of the category the research item belongs to.
        category_id : str, optional
             Id of the category the research item belongs to.
        project_id : str, optional
             Optionally to get researchitems from another project as the furthrmind sdk was initiated with

        Returns
        -------
        List[Self]
             List of instances of the researchitem class.

        Raises
        ------
        AssertionError
            ids, shortids, nor names must be provided.
            If names is provided but neither category_name nor category_id is provided.

        """

        assert ids or shortids or names, AssertionError("Either ids, shortids or names must be given")
        if names:
            assert category_name or category_id, AssertionError("Either category_name or category_id must be given")

        return cls._get_many(ids, shortids, names, category_name, category_id, project_id=project_id)

    @classmethod
    def get_all(cls, project_id: str = "") -> List[Self]:
        """
        Method to get all researchitems belonging to one project

        Parameters
        ----------
        project_id: str
            Optionally to get researchitems from another project as the furthrmind sdk was initiated with

        Returns
        -------
        List[Self]
            A list containing instances of the researchitem class.
        """

        return cls._get_all(project_id)


    @classmethod
    @BaseClass._create_instances_decorator(_fetched=False)
    def create(cls, name: str, group_name: str = "", group_id: str = "", category_name: str = "", category_id: str = "",
               project_id=None) -> Self:
        """
        Parameters
        ----------
        name : str
            The name of the item to be created.
        group_name : str, optional
            The name of the group where the new item will belong to. Group name can only be considered for groups that
            are not subgroups. Either `group_name` or `group_id` must be specified.
        group_id : str, optional
            The id of the group where the new item will belong to. Either `group_name` or `group_id` must be specified.
        category_name : str, optional
            The name of the category that the new item will belong to. Either `category_name` or `category_id` must be specified.
        category_id : str, optional
            The id of the category that the new item will belong to. Either `category_name` or `category_id` must be specified.
        project_id : object, optional
            Optionally create a researchitem in another project as the furthrmind sdk was initiated with.

        Returns
        -------
        Self
            instance of the researchitem class

        Raises
        ------
        AssertationError
            If neither group_id nor group_name is provided.
            If neither `category_name` nor `category_id` are specified.

        """

        from furthrmind.collection import Category

        assert group_name or group_id, "Either group_name or group_id must be specified"
        assert category_name or category_id, "Either category name or id must be specified"

        data = cls._prepare_data_for_create(name, group_name, group_id, project_id)

        category_dict = {}
        if category_name:
            category_dict["name"] = category_name
        if category_id:
            category_dict["id"] = category_id

        data["category"] = category_dict
        id = cls._post(data, project_id)

        if "id" not in category_dict:
            categories = Category._get_all(project_id)
            for cat in categories:
                if cat.name == category_name:
                    category_dict["id"] = cat.id
                    break

        data["id"] = id
        return data

    @classmethod
    @BaseClass._create_instances_decorator(_fetched=False)
    def create_many(cls, data_list: List[Dict], project_id: str = "") -> Self:
        """
        Parameters
        ----------
        data_list : List[Dict]
            List of dictionaries containing information about the items to be created. Each dictionary should have the
            following keys:
                - name: str
                    The name of the group to be created.
                - group_name: str
                    The name of the group where the new item will belong to. Group name can only be considered for groups
                    that are not subgroups. Either group_name or group_id must be specified.
                - group_id: int or None
                    The ID of the group where the new item will belong to. Either group_name or group_id must be specified.
                - category_name: str
                    The name of the category that the new item will belong to. Either category_name or category_id must be specified.
                - category_id: int or None
                    The ID of the category that the new item will belong to. Either category_name or category_id must be specified.
        project_id : str, optional
            Optionally to create researchitems in another project as the furthrmind sdk was initiated with

        Returns
        -------
        List[Self]
            A list of instances of the researchitem class created.

        Raises
        ------
        AssertationError
            If name not specified
            If neither category name nor ID is specified.
            If neither group_id nor group_name is provided.


        """

        new_list = []
        category_id_not_present = False

        for data in data_list:
            category_name = data.get('category_name')
            category_id = data.get('category_id')
            assert category_name or category_id, "Either category name or id must be specified"

            # raises an error if name not specified or if neither group_name nor group_id is provided
            temp_data = cls._prepare_data_for_create(data.get("name"), data.get("group_name"), data.get("group_id"),
                                                     project_id)

            category_dict = {}
            if category_name:
                category_dict["name"] = category_name
            if category_id:
                category_dict["id"] = category_id

            temp_data["category"] = category_dict
            new_list.append(temp_data)
            if not "id" in category_dict:
                category_id_not_present = True

        id_list = cls._post(new_list, project_id, force_list=True)
        category_mapping = {}
        if category_id_not_present:
            categories = Category._get_all(project_id)
            category_mapping = {cat.name: cat for cat in categories}

        for data, id in zip(new_list, id_list):
            data["id"] = id
            if "id" not in data["category"]:
                cat_id = category_mapping.get(data["category"]["name"])
                data["category"]["id"] = cat_id

        return new_list

    def add_datatable(self, name: str, columns: List[Dict], project_id: str = "" ) -> "DataTable":
        """
        Method to create a new datatable within this researchitem. Add the created datatable to the datatables attribute

        Parameters
        ----------
        name : str
            Name of the datatable.
        columns : List[Dict]
            A list of columns that should be added to the datatable. Each column is represented as a dictionary with the following keys:
            - name : str
                Name of the column.
            - type : str
                Type of the column. Either "Text" or "Numeric". Data must fit the specified type.
            - data : Union[List[Union[str, float]], pandas.Series]
                List of column values. Data must fit the specified type of the column.
                For Text columns, the items must be convertable to strings
                For Numeric columns, the items must be convertable to floats.
                Can be a list or a pandas.Series.
            - unit : dict or str
                Unit of the column. It can be represented as either a dictionary with 'id' or 'name', or a string
                representing the name or id of the unit.
        project_id : str, optional
            Optionally, specify the id of another project to create the datatable in.

        Returns
        -------
        DataTable
            An instance of the DataTable class representing the created datatable.

        """

        from furthrmind.collection import DataTable
        datatable = DataTable.create(name, researchitem_id=self.id, columns=columns, project_id=project_id)

        new_datatable = list(self.datatables)
        new_datatable.append(datatable)
        self.datatables = new_datatable

        return datatable




from functools import wraps
from types import MethodType

def furthr_wrap(force_list=False):
    def decorator(function):
        @wraps(function)
        def wrapper(*args, **kws):
            response = function(*args, **kws)
            data = None
            if response.status_code in [200, 400, 401, 403, 500]:
                data = response.json()
            
            if response.status_code != 200:
                print("error", response.status_code)
                if data is not None:
                    print("message", data["message"])
                else:
                    raise ValueError("Server returned an error")

            if data["status"] == "error":
                print("error", data["message"])
                return
            else:
                if force_list:
                    return data["results"]
                if len(data["results"]) == 1:
                    return data["results"][0]
                else:
                    return data["results"]
        return wrapper
    return decorator

def instance_overload(self, methods):
    """ Adds instance overloads for one or more classmethods"""
    for name in methods:
        if hasattr(self, name):
            setattr(self, name, MethodType(getattr(self, name).__func__, self))



import hashlib
import os
import requests
import mimetypes
from pathvalidate import sanitize_filename
import sys
from io import BytesIO


class FileLoader:
    chunkSize = 8192000

    def __init__(self, host, api_key):
        self.host = host
        self.api_key = api_key
        self.session = requests.session()
        self.session.headers.update({"X-API-KEY": api_key})

        response = self.session.get(f"{host}/s3")
        if response.status_code != 200:
            self.host = False
            print("Check if your host is correct!")
            return
        self.s3Enabled = response.json()["enabled"]

    def uploadFile(self, filePath, fileName=None, parent=None):
        if fileName == None:
            fileName = os.path.basename(filePath)
        if not self.host:
            print("Check if your host is correct!")
            return

        parameter = {
            "name": fileName
        }
        response = self.session.post(f"{self.host}/api2/file", json=parameter)
        if response.status_code != 200:
            return
        fileID = response.json()["results"][0]
        result = self.updateFile(fileID, filePath, file_name=fileName)
        if not result:
            print("Upload not successful")
            return
        print(f"Upload successful, file_id: {fileID}")
        if parent and result:
            _print = False
            if "type" not in parent:
                _print = True
            if "id" not in parent:
                _print = True
            if "project" not in parent:
                _print = True
            if _print:
                print("parent must have the following structure: {project: project_id, type: x, id: item_id}\n "
                      "The type must be one out of: experiment, sample, researchitem, corresponding to your parent")
                return fileID
            response = self.session.get(f"{self.host}/api2/project/{parent['project']}/{parent['type']}/{parent['id']}")
            item = response.json()["results"][0]
            files = item["files"]
            files.append({"id": fileID})
            parameter = {"id": item["id"],
                         "files": files}
            response = self.session.post(f"{self.host}/api2/project/{parent['project']}/{parent['type']}", json=parameter)
            if response.status_code != 200:
                print("file not attached to parent")
            print(f"File attached to parent: file_id: {fileID}")

        return fileID, fileName

    def updateFile(self, file_id, file_path, file_name=None):
        if not os.path.isfile(file_path):
            return False

        file_path = file_path.replace("\\", "/")
        if not file_name:
            file_name = file_path.split("/")[-1]

        if self.s3Enabled:
            flag = self.s3Upload(file_id, file_name, file_path)
        else:
            flag = self.chunkUpload(file_id, file_name, file_path)
        return flag

    def chunkUpload(self, fileID, fileName, filePath):
        chunkIDList = []
        with open(filePath, "rb") as uploadFile:
            data = uploadFile.read(self.chunkSize)
            url = f"{self.host}/app1/chunks/add"
            session = requests.session()
            session.headers.update({
                "X-API-KEY": self.api_key,
                'Content-Type': 'application/octet-stream',
            })
            while data:
                md5 = self.getMD5(data)
                session.headers.update({
                    "MD5": md5,
                })
                response = session.post(url, data=data)
                if response.status_code != 200:
                    return False
                chunkID = response.json()["_value"]

                chunkIDList.append(chunkID)
                data = uploadFile.read(self.chunkSize)

        parameter = {
            "id": fileID,
            "chunks": chunkIDList,
            "name": fileName
        }
        response = self.session.post(f"{self.host}/api2/file", json=parameter)
        if response.status_code != 200:
            return False
        return True

    def s3Upload(self, fileID, fileName, filePath):


        mimeType = mimetypes.MimeTypes().guess_type(filePath)[0]
        if mimeType is None:
            mimeType = "text/plain"
        payload = {"filename": fileName, "type": mimeType,
                   "metadata": {"name": fileName, "type": mimeType},
                   "createFileObject": False}  # adjust for your file

        response = self.session.post(f"{self.host}/s3/multipart", json=payload)
        if response.status_code != 200:
            return False
        uploadData = response.json()
        fileSize = os.path.getsize(filePath)      # bytes
        numberOfChunks = (fileSize // self.chunkSize) + 1
        numberOfChunksString = "1"
        for i in range(2, numberOfChunks + 1):
            numberOfChunksString = f"{numberOfChunksString},{i}"

        response = self.session.get(f"{self.host}/s3/multipart/{uploadData['uploadId']}/batch", params={
            "key": uploadData['key'], 'partNumbers': numberOfChunksString})
        if response.status_code != 200:
            return False
        batches = response.json()

        chunkNumber = 1
        partsList = []
        with open(filePath, "rb") as uploadFile:
            data = uploadFile.read(self.chunkSize)
            while data:
                etag = requests.put(batches['presignedUrls'][f"{chunkNumber}"],
                                    data=data).headers['etag']
                partsList.append({
                    "PartNumber": chunkNumber, "ETag": etag
                })
                chunkNumber += 1
                data = uploadFile.read(self.chunkSize)

        partsDict = {"parts": partsList}
        response = self.session.post(f"{self.host}/s3/multipart/{uploadData['uploadId']}/complete", params={
            'key': uploadData['key']}, json=partsDict)
        if response.status_code != 200:
            return False

        parameter = {
            "id": fileID,
            "s3key": uploadData['key'],
            "name": fileName
        }
        response = self.session.post(f"{self.host}/api2/file", json=parameter)
        if response.status_code != 200:
            return False
        return True


    def downloadFile(self, fileID, folderPath=None, bytesIO: bool=False, overwrite=False):


        response = self.session.get(f"{self.host}/api2/files/{fileID}/content")

        if not bytesIO:
            if not os.path.isdir(folderPath):
                print("This is not valid folder")
                return False, None

            file_response = self.session.get(f"{self.host}/api2/file/{fileID}")
            file = file_response.json()
            file_name = file["name"]
            file_name = sanitize_filename(file_name)
            filePath = f"{folderPath}/{file_name}"
            if os.path.isfile(filePath):
                if overwrite is False:
                    print("File already present!")
                    return False, None

            with open(filePath, "wb+") as newFile:
                for chunk in response.iter_content(chunk_size=self.chunkSize):
                    if chunk:  # filter out keep-alive new chunks
                        newFile.write(chunk)

            print(f"Download successful, file_id: {fileID}")

            return True, filePath

        else:
            bytesIO = BytesIO()
            for chunk in response.iter_content(chunk_size=self.chunkSize):
                if chunk:
                    bytesIO.write(chunk)

            print(f"Download successful, file_id: {fileID}")

            return True, bytesIO




    def getMD5(self, byteData):
        return hashlib.md5(byteData).hexdigest()

# if __name__ == "__main__":
#     argv = sys.argv[1:]
#     kwargs = {kw[0]: kw[1] for kw in [ar.split('=') for ar in argv if ar.find('=') > 0]}
#     args = [arg for arg in argv if arg.find('=') < 0]
#
#     host = kwargs.get("host")
#     api_key = kwargs.get("api_key")
#     if not host or not api_key:
#         print("Please specify host and api_key")
#         sys.exit()
#
#     fl = FileLoader(host, api_key)
#
#     file_path = kwargs.get("file_path")
#     file_id = kwargs.get("file_id")
#     file_name = kwargs.get("file_name")
#
#     parent_project = kwargs.get("parent_project")
#     parent_type = kwargs.get("parent_type")
#     parent_id = kwargs.get("parent_id")
#     parent = None
#     if parent_project and parent_type and parent_id:
#         parent = {
#             "project": parent_project,
#             "type": parent_type,
#             "id": parent_id
#         }
#     download_folder = kwargs.get("download_folder")
#     overwrite = kwargs.get("overwrite")
#     if overwrite:
#         if overwrite.lower() == "true":
#             overwrite = True
#         else:
#             overwrite = False
#     else:
#         overwrite = False
#
#
#     command = args[0]
#
#     if command == "upload":
#         file_id = fl.uploadFile(file_path, file_name, parent)
#
#     elif command == "update":
#         fl.updateFile(file_id, file_path, file_name)
#
#     elif command == "download":
#         fl.downloadFile(file_id, download_folder, overwrite=overwrite)
#
#     else:
#         print("Unknown command")

